function [w,b, obj] = train(y, mu, sigma, n, eta1, eta2, C, robust)
%[w,b, obj] = train(y, mu, sigma, n, eta1, eta2, C, robust)
% Does Robust Clustering based Linear Classification using SeDuMi
%Input:
% y - column vector containing labels of clusters
% mu - matrix whose rows are means of clusters
% sigma - column vector containing std.div. of clusters
% n - column vector containing number of points in clusters
% eta1, eta2 desired lower bound on classification accuracy of each class
% C - regularization parameter
% if robust = 1,2 then rcb1-socp, rcb2-socp formulations are solved else cb-socp is solved
%Output:
% w,b - Optimal discriminating hyperplane w'x-b=0
% obj - Value of corresponding optimal objective function
    
    posidx = find(y==1);
    negidx = find(y==-1);
    k1 = length(posidx);
    k2 = length(negidx);
    k = k1+k2;
    dim = size(mu,2);
    rconf = 0.9;
    f = chi2inv(rconf,dim+1);
%     Kappa(posidx,1)=sqrt(eta1/(1-eta1))*ones(k1,1);
    Kappa(posidx,1) = norminv(eta1);
%     Kappa(negidx,1)=sqrt(eta2/(1-eta2))*ones(k2,1);
    Kappa(negidx,1) = norminv(eta2);
    
%   Calculating s vector from sigma, etas for robustness
    if(robust==1)
        for j=1:k
            if(Kappa(j) ~= 0)
                a = 2*dim^2*n(j);
                b = dim*n(j)*sigma(j)^2*(Kappa(j)^2-4*dim);
                c = 2*dim*n(j)*sigma(j)^4*(dim-Kappa(j)^2);
                d = Kappa(j)^2*sigma(j)^6*(dim*n(j)-2*f);
                [x,nroot]=cubicfcn(a, b, c, d);
                if(isreal(x))
                    x = max(x);
                else
                    x = x(1);
                end
                sigmas(j,1) = sqrt(x);
                if(sigmas(j)-sigma(j) < -eps)
                    fprintf('Value of sigma*=%f(%f) not acceptable at %d\n',sigmas(j),sigma(j),j);
                end
            else
                sigmas(j,1) = sigma(j);
            end
            gs(j,1) = sqrt(f/(n(j)*(1+(Kappa(j)^2*sigma(j)^2/(2*dim*sigmas(j)^2)))));
        end
        s = Kappa.*sigmas+gs.*sigma;
        s(find(sigma == 0)) = 0;
    else
        if(robust==2)
            for j=1:k
                s(j,1)=(sqrt((dim*(n(j)-1)*finv(sqrt(rconf),dim,n(j)-dim))/(n(j)*(n(j)-dim)))+Kappa(j)*sqrt((n(j)-1)/chi2inv((1-sqrt(rconf))/2,n(j)-1)))*sigma(j);
            end
        else
            s = Kappa.*sigma;
        end
    end

    %%% Uncomment for SVM Solution  %%%
% % %     s = zeros(k,1);

% % %     if(dim == 2)
% % %         hold on;
% % %         axis equal;
% % %         for j=1:k
% % %             xstr = sprintf('%f+%f*cos(t)',mu(j,1),s(j));
% % %             ystr = sprintf('%f+%f*sin(t)',mu(j,2),s(j));
% % %             h=ezplot(xstr,ystr);
% % %             if(y(j) == 1)
% % %                 set(h,'color','r');
% % %             else
% % %                 set(h,'color','b');
% % %             end
% % %         end
% % %     end

    %Preparing SeDuMi variables
    cost(posidx,1) = C*k2/(k1+k2);
    cost(negidx,1) = C*k1/(k1+k2);
    m1=sum(n(posidx));m2=sum(n(negidx));cost(posidx,1) = m2;cost(negidx,1)=m1;cost=C*cost.*n/sum(cost.*n);
    At=-sparse([repmat(y,1,dim).*mu -y eye(k) -s zeros(k,1);zeros(k,dim+1) eye(k) zeros(k,2);zeros(1,dim+k+1) 1 0;eye(dim) zeros(dim,k+3);zeros(1,dim+k+2) 1;zeros(1,dim+k+3);zeros(1,dim+k+1) 1 0]);
    b = sparse([zeros(dim+1,1);-cost;0;-1]);
    c = sparse([-ones(k,1);zeros(k,1);zeros(dim+1,1);0;1;0]);
    K = [];
    K.l = 2*k;
    K.q = dim+1;
    K.r = 3;
    
    %Calling SeDuMi
    pars.fid = 0;
    CalTime=cputime;
    [soln,dual,info] = sedumi(At,b,c,K,pars);
    CalTime = cputime - CalTime;
    clear At b c K;

    %Inference from soln.
    if((info.pinf == info.dinf) & (info.pinf == 0))
        w = dual(1:dim,1);
        b = dual(dim+1);
        xi = dual(dim+2:dim+k+1,1);
        t = dual(dim+k+2);
        obj = dual(dim+k+3)+cost'*xi;
% % %         if(dim == 2)
% % %             str=sprintf('(%f)*x+(%f)*y-%f',w(1),w(2),b);
% % %             h=ezplot(str,[-1 1]);
% % %             set(h,'color','k');
% % %             str=sprintf('(%f)*x+(%f)*y-%f',w(1),w(2),b-1);
% % %             h=ezplot(str,[-2 2]);
% % %             set(h,'color','k','LineStyle','--');
% % %             str=sprintf('(%f)*x+(%f)*y-%f',w(1),w(2),b+1);
% % %             h=ezplot(str,[-2 2]);
% % %             set(h,'color','k','LineStyle','--');
% % %         end
    else
        w = 0;b = 0; obj = 0;
        fprintf('dinf=%d pinf=%d numerr=%d\n', info.dinf, info.pinf, info.numerr);
    end
    
return
